import common
import coveragepy_parser


def convert_coverage(data, report=None):
    if report:
        codecov_files = report['coverage']
        codecov_report = report
    else:
        codecov_files = {}
        codecov_report = {"coverage": codecov_files}
    for file_name in data['files']:
        if common.valid(file_name):
            file_object = data['files'][file_name]
            total_lines = coveragepy_parser.total_lines(file_object)
            codecov_file = codecov_files[file_name] if file_name in codecov_files else None
            if codecov_file is None:
                codecov_file = {}
                codecov_files[file_name] = codecov_file
            for i in range(1, total_lines):
                if str(i) in codecov_file:
                    curr = 0 if codecov_file[str(i)] is None else codecov_file[str(i)]
                    offset = 1 if i in file_object['executed_lines'] else 0
                    codecov_file[str(i)] = curr + offset
                else:
                    codecov_file[str(i)] = 1 if i in file_object['executed_lines'] else None
                if codecov_file[str(i)] is None:
                    codecov_file[str(i)] = 0 if i in file_object['missing_lines'] else None
    return codecov_report



import os

import git
import requests

url = 'https://api.codacy.com'

repo = git.Repo(os.getcwd())
master = repo.head.reference
commit = master.commit.hexsha


def send_report(reports, language):
    headers = {
        'Content-Type': 'application/json',
        'project-token': os.getenv('CODACY_PROJECT_TOKEN'),
    }
    if len(reports) == 1:
        effective_url = f'{url}/2.0/coverage/{commit}/{language.capitalized()}?partial=false'
        print(f"- Sending Codacy report to {effective_url}")
        r = requests.post(url=effective_url, headers=headers, data=reports[0])
        return r.content.decode("utf-8")
    else:
        effective_url = f'{url}/2.0/coverage/{commit}/{language}?partial=true'
        print(f"- Sending Codacy report to {effective_url}")
        for report in reports:
            r = requests.post(url=effective_url, headers=headers, data=report)
            print("- Codacy Report sent!")
            print(f"- {r.content.decode('utf-8')}")
        effective_final_url = f'{url}/2.0/commit/{commit}/coverageFinal'
        print(f"- Sending Final Codacy report to {effective_final_url}")
        final_response = requests.post(url=effective_final_url, headers=headers, json="")
        final_response.content.decode("utf-8")
    return None



import hashlib
import os

import git

import common
import coveragepy_parser

repo = git.Repo(os.getcwd())
master = repo.head.reference
commit = master.commit
commitHex = commit.hexsha


def merge_coverage(existing_file, source_file):
    target = existing_file['coverage']
    source = source_file['coverage']
    for index in range(0, len(source)):
        target[index] = source[index] if target[index] is None else target[index] + source[index]
    return target


def convert_coverage(data, report=None):
    if report:
        coveralls_files = report['source_files']
        coveralls_report = report
    else:
        coveralls_files = {}
        coveralls_report = {
            'repo_token': os.getenv('COVERALLS_REPO_TOKEN'),
            'service_name': 'local-ci'}
        head = {
            'id': commitHex,
            'author_name': commit.author.name,
            'author_email': commit.author.email,
            'committer_name': commit.committer.name,
            'committer_email': commit.committer.email,
            'message': commit.message
        }
        remotes = list(map(lambda remote: {'name': remote.name, 'url': remote.url}, repo.remotes))
        git_repo = {
            'head': head,
            'branch': repo.active_branch.name,
            'remotes': remotes
        }
        coveralls_report['git'] = git_repo
        coveralls_report['source_files'] = []
    source_files = coveralls_report['source_files']
    for file_name in data['files']:
        if common.valid(file_name):
            file_object = data['files'][file_name]
            text_file = open(file_name, "r")
            file_content_bytes = text_file.read().encode('utf-8')
            total_lines = coveragepy_parser.total_lines(file_object) + 2
            line_coverage = [None] * total_lines
            executed_lines = file_object['executed_lines']
            for entry in executed_lines:
                line_coverage[entry - 1] = 1
            missing_lines = file_object['missing_lines']
            for entry in missing_lines:
                line_coverage[entry - 1] = 0
            source_file = {
                'name': file_name,
                'source_digest': hashlib.md5(file_content_bytes).hexdigest(),
                'coverage': line_coverage,
            }
            filter_result = list(filter(lambda file: file['name'] == file_name, coveralls_files))
            existing_file = filter_result[0] if len(filter_result) > 0 else None
            if existing_file is None:
                source_files.append(source_file)
            else:
                existing_file['coverage'] = merge_coverage(existing_file, source_file)

    return coveralls_report



import os
import tempfile

import requests

temporaryFolder = tempfile.gettempdir()

url = 'https://coveralls.io/api/v1/jobs'
coveralls_file = 'coveralls.json'

coveralls_tmp_file = os.path.join(temporaryFolder, coveralls_file)


def send_report(report):
    file_save = open(coveralls_tmp_file, "w")
    file_save.write(report)
    print(f"- Saved file in {file_save.name}")
    file_save.close()
    files = {'json_file': open(coveralls_tmp_file, 'rb')}
    r = requests.post(
        url=url,
        files=files,
    )
    return r.content.decode('utf-8')



#!/usr/bin/python

import json
import os

import codacy_client
import codacy_converter
import codecov_client
import codecov_converter
import coveralls_client
import coveralls_converter

banner = """
  ______   .___  ___. .__   __.  __     .______       _______ .______     ______   .______     .___________. _______ .______
 /  __  \  |   \/   | |  \ |  | |  |    |   _  \     |   ____||   _  \   /  __  \  |   _  \    |           ||   ____||   _  \ 
|  |  |  | |  \  /  | |   \|  | |  |    |  |_)  |    |  |__   |  |_)  | |  |  |  | |  |_)  |   `---|  |----`|  |__   |  |_)  |
|  |  |  | |  |\/|  | |  . `  | |  |    |      /     |   __|  |   ___/  |  |  |  | |      /        |  |     |   __|  |      /
|  `--'  | |  |  |  | |  |\   | |  |    |  |\  \----.|  |____ |  |      |  `--'  | |  |\  \----.   |  |     |  |____ |  |\  \----.
 \______/  |__|  |__| |__| \__| |__|    | _| `._____||_______|| _|       \______/  | _| `._____|   |__|     |_______|| _| `._____|

<Experimental Python Version>

by João Esperancinha
"""

if __name__ == '__main__':
    f = open('coverage.json')
    data = json.load(f)
    f.close()

    print(banner)
    codecov_token = os.getenv('CODECOV_TOKEN')
    if codecov_token is not None:
        print("Processing Codecov reports...")
        codecov_report = codecov_converter.convert_coverage(data)
        print(codecov_client.send_report([json.dumps(codecov_report)]))
        print("Codecov reporting complete!")
    else:
        print("* CODECOV_TOKEN not configured.")

    coveralls_token = os.getenv('COVERALLS_REPO_TOKEN')
    if coveralls_token is not None:
        print("Processing Coveralls reports...")
        coveralls_report = coveralls_converter.convert_coverage(data)
        print(coveralls_client.send_report(json.dumps(coveralls_report)))
        print("Coveralls reporting complete!")
    else:
        print ("* COVERALLS_REPO_TOKEN not configured.")

    codacy_token = os.getenv('CODACY_PROJECT_TOKEN')
    if codacy_token is not None:
        print("Processing Codacy reports...")
        codacy_report = codacy_converter.convert_coverage(data)
        print(codacy_client.send_report([json.dumps(codacy_report)], codacy_converter.Language.PYTHON))
        print("Codacy reporting complete!")
    else:
        print ("* CODACY_PROJECT_TOKEN not configured.")



def total_lines(file_object):
    executed_lines = max(file_object['executed_lines']) if file_object['executed_lines'] else 0
    missing_lines = max(file_object['missing_lines']) if file_object['missing_lines'] else 0
    return max(executed_lines, missing_lines)



from enum import Enum
import common

class Language(Enum):
    JAVA = 1
    KOTLIN = 2
    SCALA = 3
    PYTHON = 4
    JAVA_SCRIPT = 5

    def capitalized(self):
        name = self.name
        return "".join(list(map(lambda t: t.capitalize(), name.split("_"))))

    def ext(self):
        if self is Language.JAVA:
            return "java"
        if self is Language.KOTLIN:
            return "kt"
        if self is Language.SCALA:
            return "scala"
        if self is Language.PYTHON:
            return "py"
        if self is Language.JAVA_SCRIPT:
            return "js"


def convert_coverage(data):
    data_files = data['files']
    total_covered = sum(map(lambda fn: len(data_files[fn]['executed_lines']), data_files))
    total_lines = sum(map(lambda fn: len(data_files[fn]['executed_lines']) + len(
        data_files[fn]['missing_lines']), data_files))
    total_percent = total_covered * 100 / total_lines
    codacy_report = {'total': int(total_percent)}
    codacy_file_reports = []
    codacy_report['fileReports'] = codacy_file_reports

    for file_name in data_files:
        if common.valid(file_name):
            file_object = data_files[file_name]
            file_covered = len(file_object['executed_lines'])
            file_lines = len(file_object['executed_lines']) + len(file_object['missing_lines'])
            codacy_file_report = {'filename': file_name, 'total': int(file_covered * 100 / file_lines)}
            line_coverage = {}
            for index in file_object['executed_lines']:
                line_coverage[str(index)] = 1
            for index in file_object['missing_lines']:
                line_coverage[str(index)] = 0
            codacy_file_reports.append(codacy_file_report)
            codacy_file_report['coverage'] = line_coverage
    return codacy_report
